"""
MediLink Scheduler: A medical claim scheduling and management system that integrates functionalities from MediScheduler and MediLink to optimize medical claim submissions based on patient deductible status, insurance provider deadlines, and a 90-day submission window. It leverages real-time data from insurance APIs for dynamic scheduling to maximize insurance payments by aligning claim submission with deductible fulfillment.

Features:
- Deductible Optimization: Determines optimal submission times based on deductible fulfillment to maximize reimbursements.
- Dynamic Scheduling: Updates claim submission timelines in response to changes in deductible status and insurance coverage data.
- Secure Data Handling: Ensures the security of sensitive patient data through encryption and HIPAA-compliant practices.
- Integration with MediLink Systems: Seamlessly formats, submits, and tracks claims across MediLink and associated systems.
- Robust Error Handling and Notifications: Implements mechanisms for error detection and user alerts regarding scheduling and submission errors.

Main Functionalities:
1. Configuration and System Initialization:
   - Configures and establishes secure connections to insurance APIs.
   - Loads encrypted system settings and initializes subsystems for logging, data encryption, and notifications.

2. Scheduler Integration:
   - Enhances MediLink's main menu to include an option for "Optimize and Schedule Submissions".
   - This option is visible only when there are new or imminent claims, activating the scheduling function for these claims.

3. Scheduling Process:
   - Identifies new or urgent claims.
   - Fetches real-time deductible information from insurance APIs.
   - Computes remaining deductible and determines suitable submission dates.
   - Schedules claims with consideration for deductible status and the 90-day deadline, updating a scheduling log with details.

4. User Dashboard Update:
   - Displays scheduled submissions, including deductible statuses and scheduled dates.
   - Allows users to manually adjust or override automated scheduling decisions.

5. Pre-Submission Checks:
   - Daily verification of claims ready for submission.
   - Prepares 837P claim files and finalizes submission details with user confirmation.
   - Securely transmits data to insurance providers and logs all submission activities.

6. Direct Submission Handling:
   - Uses MediLink_Up.submit_claims to manage final claim submissions.
   - Ensures that all scheduled claims are submitted with requisite details.

7. Data Security:
   - Applies encryption to any new data storage or sensitive data involved in the scheduling process.
   - Maintains compliance with HIPAA and relevant security standards.

8. Error Management and Alerts:
   - Extends error handling capabilities to include specific scheduling and submission issues.
   - Enhances notifications for reminders and updates on deductible status.

9. Integration with MediLink Logging and Error Handling:
   - Integrates scheduler operations into MediLink’s existing logging and error management frameworks.

10. Maintenance and Data Cleanup:
    - Regularly refreshes insurance data mappings and updates deductible information.
    - Manages cleanup of processed claims to maintain database integrity and performance.

11. User Interface Improvements:
    - Updates UI to ensure user-friendliness and provides clear operational feedback.
    - Revises help documentation to reflect new functionalities and guides for scheduling tasks.

Database Management:
- Maintains a JSON-based database for patient billing data, labeled as patient_billing_db, storing information from various sources (Z.dat file, APIs, SFTP).
- Database content includes claim statuses, deductible details, error logs, and identifiers for billing readiness.
- Facilitates efficient tracking and processing of claims, with considerations for integrating individual patient data in batched submissions.
- Stays on the local machine in a defined secure location per config, ensuring HIPAA compliance without the need for data encryption at rest.

Note: Potential for data corruption or synchronization issues due to system limitations; backup and manual verification measures are advised.

Crosswalk Example:
    "payer_id": {
        "87726": {
            "medisoft_medicare_id": [],
            "medisoft_id": [
                "320"
            ],
            "endpoint": "OPTUMEDI"
        },
        "60054": {
            "medisoft_medicare_id": [],
            "medisoft_id": [
                "369"
            ],
            "endpoint": "OPTUMEDI"
        },
        "00590": {
            "medisoft_medicare_id": [],
            "medisoft_id": [
                "317"
            ],
            "endpoint": "OPTUMEDI"
        },
        

Crosswalk add:
- submit_buffer_maria per payer_id
- submit_limit_days per payer_id
- deductible_max = $2500.
- submit limit buffer = 15 days before final.

MediLink_Scheduler logic to produce dates of submission:
deductible slows it until a max limit ($2500), then send anyway.
after maria buffer, before submit (-15?) limit day.
Maria collects pre-payment, this is a factor, needs to be somehow accounted for in the process where a quantity gets pre-assigned to a patient so we know when to submit the claim.

Date last reviewed, Insurance Type, In network, Annual Deductable Remaining, Recieved pre-payment amount, Check# or 'Cash'.

In the future those pre-payments should be either eCheck, Zelle or by CC + 5% surcharge.

"""


# JSON DB structure draft.
{
  "patients": {
    "patient_id": {
      "first_name": "John",
      "last_name": "Doe",
      "date_of_birth": "1985-07-12",
      "insurance_details": {
        "provider_id": "XYZ123",
        "policy_number": "P123456789",
        "coverage_start_date": "2020-01-01"
      },
      "contact_info": {
        "email": "john.doe@example.com",
        "phone": "555-1234"
      }
    }
  },
  "claims": {
    "claim_id": {
      "patient_id": "patient_id",
      "date_of_service": "2023-04-01",
      "status": "pending",
      "scheduled_submission_date": "2023-04-15",
      "actual_submission_date": null,
      "billing_amount": 500.00,
      "deductible_applied": 100.00,
      "covered_amount": 400.00
    }
  },
  "insurance_providers": {
    "provider_id": {
      "name": "InsuranceCorp",
      "contact_details": {
        "phone": "555-6789",
        "email": "support@insurancecorp.com"
      },
      "endpoint": "https://api.insurancecorp.com"
    }
  },
  "system_configuration": {
    "last_update": "2023-04-18",
    "backup_frequency": "daily",
    "data_encryption_key": "encrypted_key_value"
  },
  "logs": {
    "log_id": {
      "timestamp": "2023-04-18T12:34:56",
      "event_type": "error",
      "description": "Failed to connect to insurance API",
      "related_claim_id": "claim_id"
    }
  },
   "billing_queue": {
   "patient_id": "claim_id"
    },
   "submitted_837p_batches": {
   "batch_id": {
      "timestamp": "2023-04-20T09:00:00",
      "patients": ["patient_id1", "patient_id2", ...],
      "endpoint": "clearinghouse_endpoint",
      "status": "submitted"
   }
  }
}